/*
** FILE NAME:   Conv.c
**
** PROJECT ID:  
**
**
** AUTHOR:      D. Maidment
**
** DESCRIPTION: This file Generates a sine wave, applies a Hamming window 
**              applies an FFT and then returns the solution in log format
**
** MODIFICATION HISTORY:
**
**              1.00    20-NOV-95
**
*/
#include <stdio.h>
#include <math.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "dave.txt"

#define NSAM     64              /* number of samples */
#define NSAML    ((2*NSAM)-1)
#define SIZE    64
#define SAMP    NSAM/2
#define NOCORR    400
#define TWOPI     6.283185307
#define MAXSIZE   512

/***************** Exercise To Take The FFT Of Data *******************/


void 
main()
{

        
        short   fullb[NSAM], fullc[NSAM], sinea[NSAM], sineb[NSAM], sinem[NSAM], fftouta[NSAM], fftoutb[NSAM], fftoutc[NSAM], fftina[NSAM], fftinb[NSAM], fftinc[NSAM], outputa[SAMP], outputb[SAMP], outputc[SAMP], conv[NSAM], window[NSAM];
        long    no_samples, no_samp, no_corr;
        float   var, scale;
        int     res_shift, res_shiftl, n, k, len;


        len             =       NSAM;
        scale           =       1;
        res_shift       =       15;
        res_shiftl      =       0;
        var             =       32767;

        if(InitFft (MAXSIZE) != EDSP_OK)
        printf("Problem With Initialisation Of FFT");


/*** Generate Sine Wave ***/

        printf("Generating Sine Wave");
        k = len / 8;
        for(n = 0; n < len; n++)
        {
        sinea[n] = floor(120 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 16;
        for(n = 0; n < len; n++)
        {
        sineb[n] = floor(120 * sin(TWOPI * k * n / len) + 0.5);
        }

        printf("Done");

/*** Multiply The Two Signals ***/

        for(n = 0; n < len; n++)
        {
        sinem[n] = (sinea[n] * sineb[n]);
        }

/*** Generate Hamming Window ***/

        if(GenHamming( window, NSAM) != EDSP_OK)
        printf("Problem With Hamming Window");


/*** Apply Hamming Window To Data ***/

        if(VectorMult (fftina, sinem, window, NSAM, res_shift) != EDSP_OK) 
        printf("problem With VectorMult");



/*** Take FFT ***/


        if(FftReal(fftouta, sinem, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");

        if(LogMagnitude( outputa, fftouta, SAMP, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

/** Calculate Inverse FFT To Check Results **/

        if(IfftInReal( fftouta, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf(" Problem With IFFT");



        printf("FFT Done");






/*** Convolution ***/

/*** Window Sine Waves ***/

        if(VectorMult (fftinb, sinea, window, NSAM, res_shift) != EDSP_OK) 
        printf("problem With VectorMult");

        if(VectorMult (fftinc, sineb, window, NSAM, res_shift) != EDSP_OK) 
        printf("problem With VectorMult");


/*** Take FFTs Of Sine Waves ***/


        if(FftReal(fftoutb, sinea, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");


        if(FftReal(fftoutc, sineb, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");


        if(LogMagnitude( outputc, fftoutc, SAMP, scale) != EDSP_OK)
        printf("problem With LogMagnitude");


        if(LogMagnitude( outputb, fftoutb, SAMP, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

/*** Create Arrays Containing Full FFT ***/

        for(n = 32; n < 64; n++)
        {
        fullb[n] = outputb[n-32];
        fullc[n] = outputc[n-32];
        }

        for(n = 0; n < 32; n++)
        {
        fullb[n] = outputb[32-n];
        fullc[n] = outputc[32-n];
        }


/*** Perform Convolution ***/

        if(ConvCyclic (conv, fullb, fullc, NSAM, res_shiftl) != EDSP_OK)
        printf("Problem With ConvComplete");



        FreeFft();

        
}

