/*
** FILE NAME:   Conv.c
**
** PROJECT ID:  
**
**
** AUTHOR:      D. Maidment
**
** DESCRIPTION: This file Generates a sine wave, applies a Hamming window 
**              applies an FFT and then returns the solution in log format
**
** MODIFICATION HISTORY:
**
**              1.00    20-NOV-95
**
*/
#include <stdio.h>
#include <math.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "dave.txt"

#define NSAM     32              /* number of samples */
#define NSAML    ((2*SIZE)-1)
#define SIZE    64
#define SAMP    NSAM/2
#define NOCORR    400
#define TWOPI     6.283185307
#define MAXSIZE   512

/***************** Exercise To Take The FFT Of Data *******************/


void 
main()
{

        short   sinea[NSAM], sineb[NSAM], fftouta[NSAM], fftoutb[SIZE], fftoutc[NSAML], fftina[NSAM], fftinb[NSAM], fftinc[NSAML], outputa[SAMP], outputb[NSAM], outputc[NSAM], conv[NSAML], window[NSAM], windowb[NSAML];
        long    no_samples, no_samp, no_corr;
        float   varience, scale;
        int     res_shift, res_shiftl, n, k, len;


        len             =       NSAM;
        scale           =       1;
        res_shift       =       15;
        res_shiftl      =       0;

        if(InitFft (MAXSIZE) != EDSP_OK)
        printf("Problem With Initialisation Of FFT");


/*** Generate Sine Wave ***/

        printf("Generating Sine Wave");
        k = len / 8;
        for(n = 0; n < len; n++)
        {
        sinea[n] = floor(16383 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 16;
        for(n = 0; n < len; n++)
        {
        sineb[n] = floor(16383 * sin(TWOPI * k * n / len) + 0.5);
        }

        printf("Done");

/*** Generate Hamming Window ***/

        if(GenHamming( window, NSAM) != EDSP_OK)
        printf("Problem With Hamming Window");


/*** Apply Hamming Window To Data ***/

        if(VectorMult (fftina, sinea, window, NSAM, res_shift) != EDSP_OK) 
        printf("problem With VectorMult");

        if(VectorMult (fftinb, sineb, window, NSAM, res_shift) != EDSP_OK) 
        printf("problem With VectorMult");


/*** Take FFT ***/


        if(FftReal(fftouta, fftina, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");

        if(FftReal(fftoutb, fftinb, SIZE, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");


        if(LogMagnitude( outputb, fftoutb, SIZE, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

        if(LogMagnitude( outputa, fftouta, SAMP, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

        printf("FFT Done");

/*** Convolution ***/

/*** Perform Convolution ***/

        if(ConvComplete (conv, outputa, outputb, NSAM, NSAM, res_shiftl) != EDSP_OK)
        printf("Problem With ConvComplete");

/*** Apply Window ***/

        if(GenHamming( windowb, NSAML) != EDSP_OK)
        printf("Problem With Hamming Window");

        if(VectorMult (fftinc, conv, windowb, NSAML, res_shift) != EDSP_OK) 
        printf("problem With VectorMult");

/*** Take FFT ***/

        if(FftReal(fftoutc, fftinc, NSAML, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");

        if(LogMagnitude( outputc, fftoutc, NSAM, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

        printf(" Done ");

        FreeFft();

        
}

