;
; FILE NAME:    convcomp.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        ConvComplete
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This routine performs a complete convolution.
;		There are up to three stages to the calculation, depending
;		on the relative overlap of the sources to each `dot product':
;			In stage 1, zeros are inserted before the start of ix
;			In stage 2, the arrays completely overlap
;			In stage 3, zeros are added after the end of ix.
;
;		int ConvComplete(output, iw, ix, iw_size, ix_size, res_shift)
;
; PARAMETERS:	
;		short output[]:	output y
;		short iw[]    : input w
;		short ix[]    : input x
;		long iw_size  : size of iw
;		long ix_size  : size of ix
;		int res_shift :	right shift applied to each output
;

	.EXPORT	_ConvComplete
	.IMPORT	SHT

;REGISTER USAGE
;
;R1,R6 -> END OF X	;INPUT ARRAY
;R2,R5 -> W	;COEF
;R9 = MIN(IW_SIZE,IX_SIZE)
;R10= COUNT
;R11= TOTAL OUTPUT=IW_SIZE+IX_SIZE-1
;R4 -> OUTPUT
;R7 = IW_SIZE
;R12= IX_SIZE
;R13= IW_SIZE+IX_SIZE-R9
;R14= SHIFT

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_ConvComplete:	MOV.L	@(0,R15),R1	;R1=IX_SIZE
	MOV.L	@(4,R15),R2	;R2=SHIFT
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV	R2,R14		;R14=SHIFT
	MOV.L	R11,@-R15
	MOV	R1,R12
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-8,R15

;ERROR CHECKING
	MOV	#27,R0
	CMP/GT	R0,R14		;SHIFT > 27 ?
	BT	RTN1
	CMP/PZ	R14		;SHIFT < 0 ?
	BF	RTN1
	CMP/PL	R7		;IW_SIZE > 0 ?
	BF	RTN1
	CMP/PL	R12		;IX_SIZE > 0 ? 
	BT	argsOK

RTN1:	BRA	RTN
	MOV	#1,R0

	.ALIGN	4

;COMPARE THE LENGTHS, LEN2 > LEN1
argsOK:	CMP/GT	R7,R12		;IX_SIZE > IW_SIZE ?
	BT	LB11
	MOV	R5,R13
	MOV	R6,R5		;R5 -> X
	MOV	R13,R6		;R6 -> W
	MOV	R7,R9		;R9=IX_SIZE=LEN2
	BRA	donelengths
	MOV	R12,R13		;R13=IW_SIZE=LEN1

	.ALIGN	4

;R6 -> X, R5 -> W
LB11:	MOV	R12,R9		;R9=IX_SIZE=LEN2
	MOV	R7,R13		;R13=IW_SIZE=LEN1

; At this point R9 > R13	;R9=LEN2, R13=LEN1, R6 -> LONG, R5 -> SHORT
donelengths:
	SUB	R13,R9		;R9=LEN2-LEN1
	SHLL2	R14
	MOV.L	R9,@(0,R15)	;STORE LEN2-LEN1 ON STACK
	SHLL2	R14
	MOV.L	R13,@(4,R15)	;STORE LEN1 ON STACK
	MOV	R13,R9		;R9=LEN1
	MOV	SHTADDR,R2	;R2=ADDRESS OF SHIFT ROUTINE
	SHLL	R14
	ADD	R2,R14		;R14=SHIFT ROUTINE ADDRESS

;STAGE ONE
;
				;R9=LEN1
	SHLL	R9
	ADD	R4,R9		;R9=OUTPUT END ADDRESS

	MOV	#0,R10		;R10=COUNT
loop1:	ADD	#1,R10		;R10=R10+1
	MOV	R5,R1		;R1 -> LHS OF SHORT
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	MOV	R6,R2		;R2=R6 -> BEGINNING OF LONG
	AND	R10,R11		;R11=LOWER 2 BITS

	MOV	R10,R0
	SHLL	R0
	MOV	R1,R13
	SUB	R0,R13

	MOVA	LB,R0
	SHLL2	R11		;R11*4
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	CLRMAC

	.ALIGN	4
inner1:	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
LB:	CMP/EQ	R13,R1
	BF	inner1

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R14		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R5
	ADD	#2,R4		;INCREASE POINTER
	CMP/EQ	R4,R9		;FINISH ?
	BF	loop1

;STAGE TWO
;
	ADD	#-2,R5
	MOV.L	@(0,R15),R9	;R9=LEN2-LEN1=LOOP COUNT
	CMP/PL	R9
	BF	donel2

	SHLL	R9
	ADD	R4,R9		;R9=OUTPUT END ADDRESS

;				;R6 -> BEGINNING OF LONG
;				;R5 -> BEGINNING OF SHORT
loop2:	ADD	#2,R6		;LONG POINTER +2
	MOV	R6,R1		;R1 ->LHS OF LONG
	MOV	R5,R2		;R2=R5 -> BEGINNING OF SHORT
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS

	AND	R10,R11		;R11=LOWER 2 BITS
	SHLL2	R11		;R11*4

	MOV	R10,R0
	SHLL	R0
	MOV	R2,R13
	SUB	R0,R13

	MOVA	LB15,R0
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	CLRMAC

	.ALIGN	4
inner2:	MAC.W	@R1+,@R2+
	ADD	#-4,R2
	MAC.W	@R1+,@R2+
	ADD	#-4,R2
	MAC.W	@R1+,@R2+
	ADD	#-4,R2
	MAC.W	@R1+,@R2+
	ADD	#-4,R2
LB15:	CMP/EQ	R13,R2
	BF	inner2

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R14		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER
	CMP/EQ	R4,R9
	BF	loop2


;STAGE THREE
;
;				;R5 -> BEGINNING OF SHORT
;				;R10=LEN1
;				;R6 -> LHS OF LONG
donel2:	MOV.L	@(4,R15),R9	;R9=LEN1=LOOP COUNT
	ADD	#2,R6
	ADD	#-1,R9
	CMP/PL	R9
	BF	RTN3
	SHLL	R9
	ADD	R4,R9		;R9=OUTPUT END ADDRESS

loop3:	ADD	#-1,R10		;R10=R10-1
	MOV	R5,R1		;R1 -> LHS OF W
	MOV	R6,R2		;R2=R6 -> BEGINNING OF X
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	AND	R10,R11		;R11=LOWER 2 BITS

	MOV	R10,R0
	SHLL	R0
	MOV	R1,R13
	SUB	R0,R13

	SHLL2	R11		;R11*4
	MOVA	LB25,R0
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	CLRMAC

	.ALIGN	4
inner3:	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
LB25:	CMP/EQ	R13,R1
	BF	inner3

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R14		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER

	ADD	#2,R6		;LONG POINTER +2
	CMP/EQ	R4,R9
	BF	loop3

RTN3:	MOV	#0,R0		;RETURN CODE
RTN:	ADD	#8,R15
	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14

	.ALIGN	4
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC
	.END
