;
; FILE NAME:    convcycl.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        ConvCyclic
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the cyclic convolution function.
;		Each output point is calculated in two loops, with the input
;		pointers updated to give circular addressing between the two
;		loops.
;
;               int ConvCyclic(output, iw, ix, size, res_shift)
;
; PARAMETERS:
;               short output[]: output y
;               short iw[]    : input w
;               short ix[]    : input x
;               long size     : size of iw and ix
;               int res_shift : right shift applied to each output
;

	.EXPORT	_ConvCyclic
	.IMPORT	SHT

;REGISTER USAGE
;
;R2,R6 -> END OF X	;INPUT ARRAY
;R3,R5 -> W	;COEF
;R9 = MIN(IW_SIZE,IX_SIZE)
;R10= COUNT
;R11= TOTAL OUTPUT=IW_SIZE+IX_SIZE-1
;R4 -> OUTPUT
;R7 = N
;R12=
;R13=
;R14= SHIFT
;R6 -> X

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_ConvCyclic:	MOV.L	@(0,R15),R1	;R1=SHIFT
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV.L	R11,@-R15
	MOV	R1,R14		;R14=SHIFT
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15

;ERROR CHECKING
	CMP/PL	R7		;N > 0 ?
	BF	RTN1
	MOV	#27,R0
	CMP/GT	R0,R14		;SHIFT > 27 ?
	BT	RTN1
	CMP/PZ	R14		;SHIFT < 0 ?
	BT	argsOK

RTN1:	BRA	RTN
	MOV	#1,R0

; Here R9=N, R6 -> LONG (X), R5 -> SHORT (W)

	.ALIGN	4
argsOK:	MOV	R5,R3		;R5 -> BEGINNING OF W
	MOV	R7,R9		;R7=R9=N
	SHLL	R9		;R9=N*2
	ADD	R9,R3
	ADD	#-2,R3		;R3 -> END OF W

	SHLL2	R14
	SHLL2	R14
	SHLL	R14
	MOV	SHTADDR,R2	;R2=ADDRESS OF SHIFT ROUTINE
	ADD	R2,R14		;R14=SHIFT ROUTINE ADDRESS


				;R7=N
				;R9=N=LOOP COUNT

	ADD	R4,R9		;R9=OUTPUT END ADDRESS
	MOV	#0,R10		;R10=COUNT
	MOV	R5,R1		;R1 -> END OF W
outer:	ADD	#1,R10		;R10=R10+1
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	MOV	R6,R2		;R2=R6 -> BEGINNING OF X
	AND	R10,R11		;R11=LOWER 2 BITS

	MOV	R10,R0
	SHLL	R0
	MOV	R1,R13
	SUB	R0,R13		;R13=LAST VALUE IN R1

	SHLL2	R11		;R11*4
	MOVA	LB5,R0		;R0=ADDRESS OF LB
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	CLRMAC

	.ALIGN	4
loop1:	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
LB5:	CMP/EQ	R13,R1
	BF	loop1

	MOV	R7,R1		;R1=N
	SUB	R10,R1		;R1=N-R10

	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	AND	R1,R11		;R11=LOWER 2 BITS

	MOV	R1,R0
	SHLL	R0
	MOV	R3,R1		;R1 -> END OF W
	MOV	R1,R13
	SUB	R0,R13

	SHLL2	R11		;R11*4
	MOVA	LB25,R0		;R0=ADDRESS OF LB
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	NOP

	.ALIGN	4
loop2:	MAC.W	@R1+,@R2+
        ADD	#-4,R1
	MAC.W	@R1+,@R2+
        ADD	#-4,R1
	MAC.W	@R1+,@R2+
        ADD	#-4,R1
	MAC.W	@R1+,@R2+
	ADD	#-4,R1
LB25:	CMP/EQ	R13,R1
	BF	loop2


	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R14		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER

	ADD	#2,R1
	CMP/EQ	R9,R4		;FINISH ?
	BF	outer

	MOV	#0,R0		;RETURN CODE
RTN:	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14

	.ALIGN	4
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC

	.END
