/*
** FILE NAME:	convpart.c
**
** PROJECT ID:	94-0040
**
** TITLE:	ConvPartial
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine  convolves the two input arrays but does not
**              include outputs derived from elements outside the array boundaries.
**
**		int ConvPartial(output, iw, ix, iw_size, ix_size, res_shift)
** PARAMETERS:
**              short output[]      : output array y
**              const short iw[]    : input array w
**              const short ix[]    : input array x
**              long iw_size        : size of iw
**		long ix_size        : size of ix
**              int res_shift       : right shift applied to each output.
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : iw_size < 1
**                                    ix_size < 1
**		                      ix_size < iw_size
**                                    res_shift < 0
**                                    res_shift > 27
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int 
ConvPartial(short output[],
	    const short iw[],
	    const short ix[],
	    long iw_size,
	    long ix_size,
	    int res_shift)
{
    /* Variable Declaration */
    int m, i;
    double temp, temp1;

    /* Input Argument Error Checking */
    if ((iw_size < 1) || (ix_size < 1) || (ix_size < iw_size) || (res_shift < 0) || (res_shift > 27))
	return EDSP_BAD_ARG;

    for (m = 0; m < (ix_size - iw_size + 1); m++)	/* Main Loop */
    {
	/* Variable Initialisation */
	temp = 0;
	i = 0;

	while ((i < iw_size) && ((m + iw_size - 1 - i) >= 0))
	{
	    temp += (double)iw[i] * ix[m + iw_size - 1 - i];
	    i++;
	}
	/* right shifting  and rounding */
	temp1 = floor(temp / (1L << res_shift) + 0.5);

	/* Saturating */
	if (temp1 > 32767)
	    temp1 = 32767;
	else if (temp1 < -32768)
	    temp1 = -32768;
	output[m] = temp1;
    }

    return EDSP_OK;		/* Return Success */
}
