/*
** FILE NAME:	corrcycl.c
**
** PROJECT ID:	94-0040
**
** TITLE:	CorrCyclic
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine cyclically correlates the two input arrays and
**              puts the result in the output array
**
**		int CorrCyclic(ouput, iw, ix,  size, res_shift)
** PARAMETERS:
**              short output[]      : output array y
**              const short iw[]    : input array w
**              const short ix[]    : input array x
**              long size           : size of arrays N
**              int res_shift       : right shift applied to each output.
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : size < 1
**                                    res_shift < 0
**                                    res_shift > 27
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int 
CorrCyclic(short output[],
	   const short iw[],
	   const short ix[],
	   long size,
	   int res_shift)
{
    /* Variable Declaration */
    int m, i;
    double temp, temp1;

    /* Input Argument Error Checking */
    if ((res_shift < 0) || (res_shift > 27) || (size < 1))
	return EDSP_BAD_ARG;

    for (m = 0; m < size; m++)
    {
	temp = 0;		/* Variable Initialisation */

	for (i = 0; i < size; i++)
	    temp += (double)iw[i] * ix[(m + i) % size];

	/* Right shifting and rounding */
	temp1 = floor(temp / (1L << res_shift) + 0.5);

	/* Saturating */
	if (temp1 > 32767)
	    temp1 = 32767;
	else if (temp1 < -32768)
	    temp1 = -32768;
	output[m] = temp1;
    }

    return EDSP_OK;		/* Return Success */
}
