;
; FILE NAME:    correlat.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        Correlate
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the correlate function.
;		The arrays start aligned, with full width dot products
;		In stage 2 (if required) zeros are added after the end of ix
;		In stage 3 (if required) zero results are generated
;
;               int Correlate(output, iw, ix, iw_size, ix_size, no_corr, 
;			      res_shift)
;
; PARAMETERS:
;               short output[]: output y
;               short iw[]    : input w
;               short ix[]    : input x
;		long iw_size  : size of iw
;		long ix_size  : size of ix
;               long no_corr  : number of correlations to comupute
;               int res_shift : right shift applied to each output
;

	.EXPORT	_Correlate
	.IMPORT	SHT

;REGISTER USAGE
;
;R1,R6 -> END OF X
;R2,R5 -> W
;R9 = MIN(IW_SIZE,IX_SIZE)
;R10= COUNT
;R11= TOTAL OUTPUT=M
;R4 -> OUTPUT
;R7 = IW_SIZE
;R12= IX_SIZE
;R13= IW_SIZE+IX_SIZE-R9
;R14= SHIFT

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack, load stack parameters and perform error checking
_Correlate:	MOV.L	@(0,R15),R1	;R1= IX_SIZE
	MOV.L	@(4,R15),R2	;R2=TOTAL OUTPUT=M
	MOV.L	@(8,R15),R3	;R3=SHIFT
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV	R3,R14		;R14=SHIFT
	MOV.L	R12,@-R15
	MOV	R1,R12		;R12=IX_SIZE
	MOV.L	R11,@-R15
	MOV	R2,R11		;R11=TOTAL OUTPUT=M
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-8,R15

; Error checking
	MOV	#27,R0
	CMP/GT	R0,R14		;SHIFT > 27 ?
	BT	RTN1
	CMP/PZ	R14		;SHIFT < 0 ?
	BF	RTN1
	CMP/PL	R11		;M > 0 ?
	BF	RTN1
	CMP/PL	R7		;IW_SIZE > 0 ?
	BF	RTN1
	CMP/PL	R12		;IX_SIZE > 0 ? 
	BF	RTN1
	CMP/GE	R7,R12		;IX_SIZE > IW_SIZE ?
	BT	argsOK

RTN1:	BRA	RTN
	MOV	#1,R0

;Here R6 -> X, R5 -> W

	.ALIGN	4
argsOK:	MOV	R12,R9		;R9=IX_SIZE=LEN2
	MOV	R7,R13		;R13=IW_SIZE=LEN1
	MOV	R11,R7		;R7=TOTAL OUTPUT=M

; Here R9 > R13	;R9=LEN2, R13=LEN1, R6 -> LONG, R5 -> SHORT
	SUB	R13,R9		;R9=LEN2-LEN1
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	ADD	#1,R9

	AND	R13,R11		;R11=LOWER 2 BITS
	SHLL2	R14
	MOV.L	R11,@(0,R15)	;STORE COUNT

	SHLL2	R14
	MOV	SHTADDR,R3	;R3=ADDRESS OF SHIFT ROUTINE
	SHLL	R14
	ADD	R14,R3		;R3=SHIFT ROUTINE ADDRESS
	MOV	R13,R10		;R10=LEN1

	SHLL	R13
	ADD	R5,R13		;R13=LAST VALUE IN R2
	MOV.L	R13,@(4,R15)	;STORE ON STACK

	SHLL	R9
	ADD	R4,R9		;R9=OUTPUT END ADDRESS
	SHLL	R7
	ADD	R4,R7

;STAGE ONE
;
;				;R6 -> BEGINNING OF LONG
;				;R5 -> BEGINNING OF SHORT
stage1:	CLRMAC
	MOV	R6,R1		;R1 ->LHS OF LONG
	MOV	R5,R2		;R2=R5 -> BEGINNING OF SHORT


	MOV.L	@(0,R15),R11	;RETRIEVE COUNT

	MOVA	LB15,R0
	SHLL	R11		;R11*2
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	MOV.L	@(4,R15),R12

	.ALIGN	4
inner1:	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
LB15:	CMP/EQ	R12,R2
	BF	inner1

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R3		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R6		;LONG POINTER +2
	ADD	#2,R4		;INCREASE POINTER

	CMP/EQ	R7,R4		;NO OF OUTPUT = M ?
	BT	done		;RETURN IF YES

	CMP/EQ	R9,R4		;LOOP COUNT=0 ?
	BF	stage1


;STAGE TWO
;
;				;R5 -> BEGINNING OF SHORT
;				;R10=LEN1
;				;R6 -> LHS OF LONG
	MOV	R10,R9		;R9=LEN1=LOOP COUNT
	SHLL	R9
	ADD	R4,R9		;R9=OUTPUT END ADDRESS

stage2:	ADD	#-1,R10		;R10=R10-1
	MOV	R5,R1		;R1 -> LHS OF W
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	MOV	R6,R2		;R2=R6 -> BEGINNING OF X
	AND	R10,R11		;R11=LOWER 2 BITS

	MOV.L	R10,R12
	SHLL	R12

	MOVA	LB25,R0
	CLRMAC
	SHLL	R11		;R11*2
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	ADD	R5,R12		;R12=LAST VALUE IN R1

	.ALIGN	4
inner2:	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
LB25:	CMP/EQ	R12,R1
	BF	inner2

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R3		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R6		;LONG POINTER +2
	ADD	#2,R4		;INCREASE POINTER
	CMP/EQ	R7,R4
	BT	done

	CMP/EQ	R9,R4
	BF	stage2

; Store any trailing zeros
	MOV	#0,R8
stage3:	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER
	CMP/EQ	R7,R4
	BF	stage3

done:	MOV	#0,R0		;RETURN CODE
RTN:	ADD	#8,R15
	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14

	.ALIGN	4
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC

	.END
