/*
** FILE NAME:	correlat.c
**
** PROJECT ID:	94-0040
**
** TITLE:	Correlate
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine correlates the two input arrays and puts the
**              result in the output array.
**
**		int Correlate(output, iw, ix, iw_size, ix_size, no_corr, res_shift)
** PARAMETERS:
**              short output[]      : output array y
**              const short iw[]    : input array w
**              const short ix[]    : input array x
**              long iw_size        : size of array w
**              long ix_size        : size of array x
**              long no_corr	    : number of correlations M to compute.
**              int res_shift       : right shift applied to each output.
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : iw_size < 1
**                                    ix_size < 1
**		                      ix_size < iw_size
**                                    no_corr < 1
**                                    res_shift < 0
**                                    res_shift > 27
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int 
Correlate(short output[],
	  const short iw[],
	  const short ix[],
	  long iw_size,
	  long ix_size,
	  long no_corr,
	  int res_shift)
{
    int m, i;
    double temp, temp1;

    /* Input Argument Error Checking */
    if ((iw_size < 1) || (ix_size < 1) || (ix_size < iw_size) ||
	(no_corr < 1) || (res_shift < 0) || (res_shift > 27))
	return EDSP_BAD_ARG;

    for (m = 0; m < no_corr; m++)	/* Main Loop */
    {
	/* Variable Initialisation */
	temp = 0;

	for (i = 0; i < iw_size; i++)
	    if ((m + i) < ix_size)
		temp += (double)iw[i] * ix[m + i];

	/* Right shifting and rounding */
	temp1 = floor(temp / (1L << res_shift) + 0.5);

	/* Saturation */
	if (temp1 > 32767)
	    temp1 = 32767;
	else if (temp1 < -32768)
	    temp1 = -32768;
	output[m] = temp1;
    }

    return EDSP_OK;		/* Return Success */
}
