;
; FILE NAME:    diir.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        DIir
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the double precision IIR filter.
;
;               int DIir(output, input, no_samples, coeff, no_sections,
;                        workspace)
;
; PARAMETERS:   short   output[]
;         const short   input[]
;               long    no_samples
;         const long    coeff[]
;               long    no_sections
;               long    *workspace
;

	.EXPORT	_DIir
	.IMPORT	MULT
	.IMPORT	DADD

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_DIir:	MOV	@(0,R15),R1	;R1=NO_SEC
	MOV	@(4,R15),R3	;R3 -> WORKSPACE
	MOV	R14,@-R15
	MOV	R13,@-R15
	MOV	R12,@-R15
	MOV	R11,@-R15
	MOV	R10,@-R15
	MOV	R9,@-R15
	MOV	R1,R9		;R9=NO_SEC
	MOV	R8,@-R15
	MOV	R9,R10		;R9=R10=N_SEC
	STS.L	MACH,@-R15
	MOV	R7,R14		;R7=R14=COEF
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-28,R15
;				;R6=NO
;				;R5=INPUT
;				;R4=OUTPUT

; Check parameters
	CMP/PL	R6		;NO >0 ?
	BF	ERR
	CMP/PL	R9		;N_SEC > 0 ?
	BT	argsOK
ERR:	BRA	RTN
	MOV	#1,R0

	.ALIGN	4
argsOK:	MOV	R3,@(0,R15)	;STORE R3=WS ON STACK
	MOV	#32,R2
	MOV	R7,@(8,R15)	;STORE COEF ON STACK
	NOP
	MOV	R9,@(12,R15)	;STORE N_SEC ON STACK

; Check that the biquad a0k's (ie result shift values) are valid
	MOV	R9,R1		;R1=NO OF SECTIONS
	ADD	#-1,R1
	CMP/PL	R1
	BF	LASTCHK
SHTCHK:	MOV.L	@R14+,R0	;READ res_shift
	CMP/PZ	R0		;res_shift >= 0 ?
	BF	ERR		;ERROR IF NOT
	CMP/GT	R2,R0		;res_shift > 32 ?
	BT	ERR		;ERROR IF YES
	ADD	#20,R14
	ADD	#-1,R1
	CMP/PL	R1
	BT	SHTCHK
LASTCHK:	MOV.L	@R14+,R0	;READ res_shift
	CMP/PZ	R0		;res_shift >= 0 ?
	BF	ERR		;ERROR IF NOT
	MOV	#48,R2
	CMP/GT	R2,R0		;res_shift > 48 ?
	BT	ERR		;ERROR IF YES

	NOP
	MOV	R7,R14		;R7=R14=COEF
newsample:
	MOV.W	@R5+,R12	;READ INPUT SAMPLE
	SHLL16	R12		;INPUT SAMPLE << 16
	MOV.L	R12,@(24,R15)	;STORE INPUT SAMPLE

; The filter is computed one biquad at a time, with 64 bit accumulators.
newbiquad:
	MOV.L	@R14+,R0	;READ res_shift
	MOV.L	R0,@(16,R15)	;STORE ON STACK
	MOV.L	@R3+,R1		;READ D(1)
	BSR	MULT		;D(1)*A(1)
	MOV.L	@R14+,R2	;READ A(1)

	SHLL	R2
	ROTCL	R1		;GET RID OF EXTRA SIGN BIT
	MOV	R1,R7		;R7=MSB
	MOV	R2,R9		;R9=LSB
	MOV.L	@R3+,R1		;READ D(2)
	BSR	MULT		;D(2)*A(2)
	MOV.L	@R14+,R2	;READ A(2)

	SHLL	R2
	BSR	DADD		;(R7,R9)=(R7,R9)+(R1,R2)
	ROTCL	R1		;GET RID OF EXTRA SIGN BIT
	MOV.L	ROUND,R2	;R2=ROUNDING
	BSR	DADD
	MOV.L	@(24,R15),R1	;READ INPUT SAMPLE FROM STACK

	MOV	R7,R1
	MOV.L	R1,@(20,R15)	;STORE D(0) ON STACK
	ADD	#-8,R3		;R3 -> D(1)
	BSR	MULT		;D(0)*B(0)
	MOV.L	@R14+,R2	;READ B(0)

	MOV	R1,R7
	MOV	R2,R9
	MOV.L	@R3+,R1		;READ D(1)
	BSR	MULT		;D(1)*B(1)
	MOV.L	@R14+,R2	;READ B(1)

	BSR	DADD		;(R7,R9)=(R7,R9)+(R1,R2)
	NOP

LB71:	MOV.L	@R3+,R1		;READ D(2)
	BSR	MULT		;D(2)*B(2)
	MOV.L	@R14+,R2	;READ B(2)

	BSR	DADD		;(R7,R9)=(R7,R9)+(R1,R2)
	NOP

LB15:	ADD	#-8,R3		;R3 -> D(1)
	MOV.L	@R3,R11		;R11=D(1)
	MOV.L	@(20,R15),R8	;READ D(0) FROM STACK
	MOV.L	R8,@R3		;UPDATE D(1)
	ADD	#4,R3
	MOV.L	R11,@R3		;D(1)=> D(2)

;CALL SHIFTING ROUTINE (defined at the end of this file)
; Special fast shifts are provided for result shifts of 0, 16, 32, 33 or 48
; bits - otherwise generic slow shifting subroutines are used.
;
	MOV	R7,R13
	MOV	R9,R8
	MOV.L	@(16,R15),R0	;READ res_shift
SHT:	CMP/EQ	#0,R0
	BT	SHT0		;CALL 0 SHIFT
	CMP/EQ	#16,R0
	BT	SHT16		;CALL 16 SHIFT
	CMP/EQ	#32,R0
	BT	SHT32		;CALL 32 SHIFT
	CMP/EQ	#33,R0
	BT	SHT33		;CALL 33 SHIFT
	CMP/EQ	#48,R0
	BT	SHT48		;CALL 48 SHIFT
	MOV	#16,R2
	CMP/GT	R2,R0
	BF	SHT1_15		;CALL 1-15 SHIFT
	MOV	#32,R2
	CMP/GT	R2,R0
	BF	SHT17_31	;CALL 17-31 SHIFT
	BRA	SHT34_47	;CALL 34-47 SHIFT
	NOP

doneshift:
	MOV.L	R8,@(24,R15)	;STORE R8 ON STACK FOR NEXT INPUT
	ADD	#4,R3		;R3 -> D(1) OF NEXT STAGE
	ADD	#-1,R10		;DECREASE STAGE COUNTER
	CMP/PL	R10
	BT	newbiquad

; 32 bit intermediate values are used between biquads. However, the output
; is a sample of only the 16 LSB's
	MOV.W	MAX,R12
	CMP/GE	R12,R8		;R8 > H'7FFF ?
	BT	CLIP
	MOV.W	MIN,R12
	CMP/GE	R8,R12		;R8 < H'8000 ?
	BF	OUT
CLIP:	MOV	R12,R8
OUT:	MOV.W	R8,@R4		;OUTPUT LOWER 16 BITS AS RESULT


; Have finished a sample, so move on to the next one
	ADD	#2,R4		;INCREASE POINTER
	MOV.L	@(8,R15),R14	;RESUME POINTER FOR COEF
	MOV.L	@(0,R15),R3	;RESUME POINTER FOR DELAY ELEMENT
	MOV.L	@(12,R15),R10	;RESUME STAGE COUNT
	ADD	#-1,R6		;DECREASE INPUT NO COUNT
	CMP/PL	R6
	BT	newsample		;START WITH ANOTHER INPUT SAMPLE

	MOV	#0,R0		;RETURN CODE
RTN:	ADD	#28,R15
	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14

	.ALIGN	4
POS	.data.l	H'7FFFFFFF
NEG	.data.l	H'80000000
ROUND	.data.l	H'40000000
ROUND1	.data.l	H'C0000000
RND	.data.w	H'4000
MAX	.data.w	H'7FFF
MIN	.data.w	H'8000

; The remainder of this file give the 64 to 32 bit shift and saturate functions.
; In each case the source is in registers (R8,R13).
; The result is generated in register R8.

;0 SHIFT
	.ALIGN	4
SHT0:	CMP/PZ	R8
	BT	LB77
	NOT	R13,R0
	MOV.L	CHK0,R2		;CHK0=H'FFFFFFFF
	TST	R2,R0
	BF	SAT1
	BRA	doneshift
	NOP
LB77:	MOV.L	CHK0,R2		;CHK0=H'FFFFFFFF
	TST	R2,R13
	BF	SAT1
	BRA	doneshift
	NOP

;16 SHIFT
	.ALIGN	4
SHT16:	MOV.L	ROUND16,R2
	CLRT
	ADDC	R2,R8
	BF	LB78
	MOV	#1,R2
	ADDV	R2,R13
	BT	SAT		;MSB'S OVERFLOW
LB78:	EXTS.W	R13,R2
	CMP/EQ	R2,R13
	BF	SAT1
	BRA	doneshift
	XTRCT	R13,R8

;32 SHIFT
	.ALIGN	4
SHT32:	MOV	ROUND32,R2
	CLRT
	ADDC	R2,R8
	MOVT	R2
	ADDV	R2,R13
	BT	SAT
	BRA	doneshift
	MOV	R13,R8

;33 SHIFT
	.ALIGN	4
SHT33:	MOV	#1,R8
	ADDV	R13,R8		;ADD ROUNDING
	BF	SHT33A		;OVERFLOW ?
	MOV.L	POS3,R8		;R8=H'7FFFFFFF >> 1 =H'40000000
	BRA	doneshift
	NOP
SHT33A:	BRA	doneshift
	SHAR	R8

;48 SHIFT
	.ALIGN	4
SHT48:	MOV.L	ROUND16,R8
	ADDV	R13,R8		;ADD ROUNDING
	BF	SHT48A		;OVERFLOW ?
	MOV.L	POS1,R8		;R8=H'7FFFFFFF
	BRA	doneshift
	NOP
SHT48A:	SHLR16	R8
	BRA	doneshift
	EXTS.W	R8,R8

;34-47 SHIFT
	.ALIGN	4
SHT34_47:	ADD	#-33,R0
	CMP/EQ	#0,R0
	BT	RND1
SHT34_47A:	SHAR	R13
	ADD	#-1,R0
	CMP/EQ	#0,R0
	BF	SHT34_47A

RND1:	MOV	#1,R8
	ADD	R13,R8
	BRA	doneshift		;RETURN
	SHAR	R8

;1-15 SHIFT AND 17-31 SHIFT
	.ALIGN	4
SHT17_31:	XTRCT	R13,R8	;EXTRACT 16-47 BIT
	SHLR16	R13		;R13 >> 16
	EXTS.W	R13,R13		;SIGN EXTENDED
	ADD	#-16,R0		;SHIFT-16
SHT1_15:	ADD	#-1,R0	;SHIFT-1
	CMP/EQ	#0,R0		;SHIFT = 0 ?
	BT	LB79
	SHAR	R13		;SHIFT BIT ACROSS R13 AND R8
	BRA	SHT1_15
	ROTCR	R8
LB79:	SHAR	R13		;ONE MORE BIT SHIFT
	ROTCR	R8
	MOV	#0,R2		;R2=0
	BF	LB80		;T = 0 ?
	ADDC	R2,R8		;NO, ADD T TO R8 AS ROUNDING
	MOVT	R2
	ADDV	R2,R13		;CARRY PROPAGATE
	BT	SAT		;MSB'S OVERFLOW ?
LB80:	CMP/PZ	R8		;R8 >= 0 ?
	MOV	#0,R2
	BT	LB81		;R2=H'00000000=SIGN OF R8
	NOT	R2,R2		;R2=H'FFFFFFFF=SIGN OF R8
LB81:	CMP/EQ	R2,R13		;IS R13 CONTAINS SIGN ONLY ?
	BF	SAT1		;SATURATE IF NOT
	BRA	doneshift		;RETURN
	NOP

;SATURATION
;
SAT:	NEGC	R13,R13
SAT1:	MOV.L	SATVAL,R8
	CMP/PZ	R13		; T=1 if +ve, 0 if -ve
	BRA	doneshift
	NEGC	R8,R8

	.ALIGN	4
ROUND16	.data.l	H'00008000
CHK0	.data.l	H'FFFFFFFF
ROUND32	.data.l	H'80000000
SATVAL	.data.l H'80000000
F16	.data.l	H'FFFFFFFF
POS1	.data.l	H'7FFFFFFF
NEG1	.data.l	H'80000000
POS3	.data.l	H'40000000
NEG3	.data.l	H'C0000000
SATVAL1	.data.w	H'8000
POS2	.data.w	H'7FFF
	.END
