/*
** Copyright Ensigma Ltd. 1995
**
** Unauthorised copying or distribution of this software is prohibited.
**
** Version 1.0.
**
*/

/*
** ENSIGMA DSP Library
*/

/* Constants */

#define BIQUAD_SIZE     6       /* no. coefficients in an IIR biquad */

#define EDSP_OK         0
#define EDSP_BAD_ARG    1
#define EDSP_NO_HEAP    2

#define EFFTALLSCALE    (-1L)
#define EFFTMIDSCALE    0x55555555L
#define EFFTNOSCALE     0x00000000L

/*
** Fourier transforms
*/
int FftComplex(short output[], const short input[], long size, long scale);
int FftInComplex(short data[], long size, long scale);
int FftReal(short output[], const short input[], long size, long scale);
int FftInReal(short data[], long size, long scale);
int IfftComplex(short output[], const short input[], long size, long scale);
int IfftInComplex(short data[], long size, long scale);
int IfftReal(short output[], const short input[], long size, long scale);
int IfftInReal(short data[], long size, long scale);
int InitFft(long max_size);
int LogMagnitude(short output[], const short input[], long no_elements, float fscale);

/*
** These three variables are for the internal use of the library FFT
** functions only. They should not be accessed by user programs.
*/
extern long max_fft_size;
extern short *bvtble, *twtble;



/*
** Window functions
*/
int GenBlackman(short data[], long N);
int GenHamming(short data[], long N);
int GenHanning(short data[], long N);
int GenTriangle(short data[], long N);

/*
** Filters
*/
int InitFir(short **workspace, long order);
int InitIir(short **workspace, long no_sections);
int InitDIir(long **workspace, long no_sections);
int InitLms(short **workspace, long order);
int Fir(short op[],
	const short ip[],
	long no_samples,
	const short coeff[],
	long order,
	int res_shift,
	short *workspace);
int Fir1(short *op_ptr,
	 short input,
	 const short coeff[],
	 long order,
	 int res_shift,
	 short *workspace);
int Iir(short op[],
	const short ip[],
	long no_samples,
	const short coeff[],
	long no_sections,
	short *workspace);
int Iir1(short *op_ptr,
	 short input,
	 const short coeff[],
	 long no_sections,
	 short *workspace);
int DIir(short op[],
	 const short ip[],
	 long no_samples,
	 const long coeff[],
	 long no_sections,
	 long *workspace);
int DIir1(short *op_ptr,
	  short input,
	  const long coeff[],
	  long no_sections,
	  long *workspace);
int Lms(short op[],
	const short ip[],
	const short ref[],
	long no_samples,
	short coeff[],
	long order,
	int res_shift,
	short mu,
	short *workspace);
int Lms1(short *op_ptr,
	 short input,
	 short ref_op,
	 short coeff[],
	 long order,
	 int res_shift,
	 short mu,
	 short *workspace);

/*
** Convolution and Correlation
*/
int ConvComplete(short op[],
		 const short iw[],
		 const short ix[],
		 long iw_size,
		 long ix_size,
		 int res_shift);
int ConvCyclic(short op[],
	       const short iw[],
	       const short ix[],
	       long size,
	       int res_shift);
int ConvPartial(short op[],
		const short iw[],
		const short ix[],
		long iw_size,
		long ix_size,
		int res_shift);
int Correlate(short op[],
	      const short iw[],
	      const short ix[],
	      long iw_size,
	      long ix_size,
	      long no_corr,
	      int res_shift);
int CorrCyclic(short op[],
	       const short iw[],
	       const short ix[],
	       long size,
	       int res_shift);


/*
** Miscellaneous
*/
int GenGWnoise(short output[], long no_samples, float variance);

int MinI(short **min_ptr, short buffer[], long buff_len);
int MaxI(short **max_ptr, short buffer[], long buff_len);
int PeakI(short **peak_ptr, short buffer[], long buff_len);

int Mean(short *mean_ptr, const short buffer[], long buff_len);
int Variance(long *variance_ptr,
	     short *mean_ptr,
	     const short buffer[],
	     long buff_len);

int VectorMult(short dest[],
	       const short matrix1[],
	       const short matrix2[],
	       long length,
	       int res_shift);
int MatrixMult(void *matrix3,
	       const void *matrix1,
	       const void *matrix2,
	       long m,
	       long n,
	       long p,
	       int res_shift);
int MsPower(long *pow_ptr, const short input[], long buff_size);
