;
; FILE NAME:    fir.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        Fir
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the FIR filter.
;
;		int Fir(output, input, no_samples, coeff, no_coeffs, res_shift,
;			workspace)
;
; PARAMETERS:	short	output[]
;	  const short	input[]
;		long	no_samples
;	  const	short	coeff[]
;		long	no_coeffs
;		int	res_shift
;		short 	*workspace
;

	.EXPORT	_Fir
	.IMPORT	SHT

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_Fir:	MOV.L	@(0,R15),R1	;R1=NO_COEFF
	MOV.L	@(4,R15),R0	;R0=res_shift
	MOV.L	@(8,R15),R3	;R3 -> WORKSPACE
	MOV.L	R9,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R10,@-R15
	MOV	R1,R9		;R9=NO_COEFF
	MOV.L	R14,@-R15
	MOV	R9,R10		;R9=R10=N_COEFF
	MOV.L	R13,@-R15
	MOV	R9,R14
	MOV.L	R12,@-R15
	SHLL	R14		;N_COEFF*2
	MOV.L	R11,@-R15
	ADD	R3,R14		;R14 -> LAST OF WORKSPACE -2
	MOV.L	R8,@-R15
	ADD	#4,R14		;R14 -> LAST OF WORKSPACE +2
	STS.L	MACH,@-R15
	MOV	R7,R8		;R7=R8=COEFF
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-4,R15
;				; Now:
;				;R6=N0
;				;R5=INPUT
;				;R4=OUTPUT

; ERROR CHECKING
	CMP/PL	R6		;NO >0 ?
	BF	ERR
	CMP/PZ	R0		;res_shift >=0 ?
	BF	ERR
	MOV	#27,R1
	CMP/GT	R1,R0		;res_shift > 27 ?
	BT	ERR
	MOV	#2,R1
	CMP/GT	R1,R9		;N_COEFF > 2 ?
	BT	argsOK

ERR:	BRA	RTN
	MOV	#1,R0

	.ALIGN	4
argsOK:	MOV.W	NONSAT,R2
	STC	SR,R1
	AND	R2,R1		;SET S BIT TO 0
	LDC	R1,SR		; - ie non-saturating arithmetic

	SHLL2	R0
	SHLL2	R0
	MOV	SHTADDR,R2	;R2=ADDRESS OF SHIFT ROUTINE
	SHLL	R0
	MOV.L	@R3+,R1		;R1=CURRENT POSITION IN CIRCULAR DELAY LINE
	ADD	R2,R0		;R0=SHIFT ROUTINE ADDRESS

; A circular delay buffer is implemented as two halves.
; First check whether we are already at the end of the array
newsample:
	CMP/EQ	R1,R3		;CURRENT POS = FIRST ?
	BF	LB50		;NO
	MOV	R14,R1		;YES, RESET TO LAST+2
LB50:	MOV	R7,R8		;R8=R7 -> COEFF
	MOV.W	@R5+,R2		;READ INPUT DATA
	MOV	R14,R13		;R13=R14=LAST+2
	MOV.W	R2,@-R1		;I/P => CURRENT POS
	MOV	R1,R2		;R2=R1 -> CURRENT POS

;				;N_COEFF=N_LOW+N_UP
;				;N_UP=LAST POS-CURRENT POS+1
	SUB	R1,R13		;GET N_UP
	SHLR	R13		;/2
	MOV.L	R13,@(0,R15)	;STORE N_UP

	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	MOV	LOWMASK,R12	;MASK AWAY LOWER 2 BITS
	AND	R13,R11		;R11=LOWER 2 BITS
	AND	R13,R12		;R12=UPPER 30 BITS

; Calculate dot product of the first half of the circular buffer
	MOV	half1ad,R13	;R13=ADDRESS OF half1
	SHLL	R11		;R11*2
	SUB	R11,R13		;R13=ADDRESS OF half1 - R11*2
	JMP	@R13		;R13=JUMP ADDRESS
	CLRMAC			;CLEAR MAC

NONSAT:	.data.w	H'0FFFD

	.ALIGN	4
	MAC	@R2+,@R8+	;MULTIPLY DATA IN WORKSPACE AND COEFF
	MAC	@R2+,@R8+
	MAC	@R2+,@R8+

half1:	CMP/PL	R12		;R12 > 0 ?
	BF	doneloop1
loop1:	MAC	@R2+,@R8+
	MAC	@R2+,@R8+
	ADD	#-4,R12
	MAC	@R2+,@R8+
	CMP/PL	R12
	MAC	@R2+,@R8+
	BT	loop1

doneloop1:
	MOV.L	@(0,R15),R11	;POP N_UP
	MOV	R9,R2		;R2=N_COEFF
	SUB	R11,R2		;N_LOW=N_COEFF-N_UP
	MOV	UPMASK,R11	;MASK UPPER 30 BITS
	MOV	LOWMASK,R12	;MASK LOWER 2 BITS
	AND	R2,R11		;R11= LOWER 2 BITS
	AND	R2,R12		;R12= UPPER 30 BITS

; Do the second half of the dot product
LB10:	SHLL	R11
	MOV	half2ad,R13	;R13= ADDRESS OF half2
	SUB	R11,R13
	JMP	@R13		;R13=JUMP ADDRESS
	MOV	R3,R2		;R2=LAST, RESET TO FIRST

	.ALIGN	4
	MAC	@R2+,@R8+
	MAC	@R2+,@R8+
	MAC	@R2+,@R8+

half2:	CMP/PL	R12
	BF	doneloop2
loop2:	MAC	@R2+,@R8+
	MAC	@R2+,@R8+
	ADD	#-4,R12
	MAC	@R2+,@R8+
	CMP/PL	R12
	MAC	@R2+,@R8+
	BT	loop2

; Call the shift routine
doneloop2:
	STS	MACH,R13	;R13=MSB'S
	JSR	@R0		;CALL SHIFT ROUTINE
	STS	MACL,R8		;R8=LSB'S

END1:	MOV.W	R8,@R4		;STORE OUTPUT
	ADD	#-1,R6		;DECREASE LOOP COUNTER
	ADD	#2,R4		;INCREASE OUTPUT POINTER
	CMP/PL	R6
	BT	newsample

; Have now processed all samples
	MOV	#0,R0		;RETURN CODE
	MOV.L	R1,@-R3		;SAVE CURRENT POSITION IN WORKSPACE

RTN:	ADD	#4,R15
	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	MOV	@R15+,R14
	MOV	@R15+,R10
	RTS
	MOV	@R15+,R9


	.ALIGN	4
half1ad	.data.l	half1
half2ad	.data.l	half2
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC

	.END
