/*
** FILE NAME:   FIRFILT.C
**
** PROJECT ID:  DSPLIB SH  
**
**
** AUTHOR:      D. Maidment
**
** DESCRIPTION: This file Generates a series of sine waves, applies an FFT 
**              to show the spectrum, then applies an FIR to remove all but one signal 
**              
**
** MODIFICATION HISTORY:
**
**              1.00    08-DEC-95
**
*/
#include <stdio.h>
#include <math.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "coeff.dat"

#define NSAM     128              /* number of samples */
#define TWOPI     6.283185307
#define no_coeff  32
#define MAXSIZE   NSAM
#define NSAMS   (NSAM/2)
#define MAXSUM  67108864

/***************** Exercise To Take The FFT Of Data *******************/


void 
main()
{
        short sine1[NSAM], sine2[NSAM], sine3[NSAM], sine4[NSAM], sine5[NSAM], signal[NSAM], fftout1[NSAM], output1[NSAMS], filout[NSAM], fftout2[NSAM], output2[NSAMS];
        int     len, n, k, res_shift;
        float   scale;
        short   *work, *locate;
        long    sum;

        scale   =   1;
        len     =   NSAM;
        res_shift = 16;
        locate = coeff;
        sum     =   0;

/*** Check Scaling Of Coefficients ***/

        for(n = 0; n < no_coeff; n++)
        {
        sum += fabs(coeff[n]);
        }

        if(sum >= MAXSUM)
        printf("Bad Coefficients");

/*** Generate Sine Wave ***/

        if(InitFft( MAXSIZE) != EDSP_OK)
        printf("Problem With Memory Allocation");

        printf("Generating Sine Wave");
        k = len / 8;
        for(n = 0; n < len; n++)
        {
        sine1[n] = floor(3276 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 16;
        for(n = 0; n < len; n++)
        {
        sine2[n] = floor(3276 * sin(TWOPI * k * n / len) + 0.5);
        }

        k = len / 4;
        for(n = 0; n < len; n++)
        {
        sine3[n] = floor(3276 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 32;
        for(n = 0; n < len; n++)
        {
        sine4[n] = floor(3276 * sin(TWOPI * k * n / len) + 0.5);
        }

        k = len / 64;
        for(n = 0; n < len; n++)
        {
        sine5[n] = floor(3276 * sin(TWOPI * k * n / len) + 0.5);
        }

        printf("Done");
        


/*** Create Signal To Be Filtered By Adding Sine waves ***/

        for(n = 0; n < len; n++)
        {
        signal[n] = (sine1[n] + sine2[n] + sine3[n] + sine4[n] + sine5[n]);
        }

/*** Perform Convolution ***/

        if(ConvComplete(filout, signal, coeff, NSAM, no_coeff, res_shift) != EDSP_OK)
        printf(" ConvComplete Not Working");


/*** Take FFT Of Input Signal ***/

        if(FftReal( fftout1, signal, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With Real FFT");

        if(LogMagnitude( output1, fftout1, NSAMS, scale) != EDSP_OK)
        printf("Problem With LogMagnetude");

/*** Begin Filter Operation ***/


        if(InitFir(&work, no_coeff) != EDSP_OK)
        printf("Problem With InitFir");

/*** Vary res_shift ***/

        if(Fir( filout, signal, NSAM, coeff, no_coeff, res_shift, work) != EDSP_OK)
        printf("Filter Problem");


/*** Perform FFT ***/

        if(FftReal( fftout2, filout, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With Real FFT");

        if(LogMagnitude( output2, fftout2, NSAMS, scale) != EDSP_OK)
        printf("Problem With LogMagnetude");

/** Fir Filter One Sample At A Time Using FIR1 **/        

        if(InitFir(&work, no_coeff) != EDSP_OK)
        printf("Problem With InitFir");

        for(n = 0; n < NSAM; n++)
        {
        if(Fir1( filout, signal[n], coeff, no_coeff, res_shift, work) != EDSP_OK)
        printf("Filter Problem");
        }

}


