/*
** FILE NAME:	Iffticom.c
**
** PROJECT ID:	94-0040
**
** TITLE:	IfftInComplex
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This file contains the wrapper used to invoke an inverse,
**		in-place, complex FFT.
**
**		int IfftInComplex(data, size, scale)
**
** PARAMETERS:
**              short data[]        : Complex  data
**              long size           : size of FFT
**              long scale          : scaling specification
**             				No scaling = 0
**                                      Complete scaling = 0xffffffff
**
** RETURNS :
**              EDSP_OK       : success
**              EDSP_BAD_ARG  : size < 2
**                              size not a power of 2.
**                              size > max_fft_size
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>
extern int FftCore(short data[], long size, long scale);

int 
IfftInComplex(short data[], long size, long scale)
{
    /* Variable declaration */
    int i;
    int test;

    /* Input Argument error testing */
    if ((size < 4) || (size > max_fft_size))
	return EDSP_BAD_ARG;

    /* test for power of 2 */
    for (test = size; test > 2; test /= 2)
	if (test & 1)
	    return EDSP_BAD_ARG;

    /* Complex Conj. */
    for (i = 0; i < size; i++)
	data[2 * i + 1] = -data[2 * i + 1];

    /*
     * The main FFT processing is performed in FftCore
     */
    if ((i = FftCore(data, size, scale)) != EDSP_OK)
	return EDSP_BAD_ARG;

    /* Complex Conj. */
    for (i = 0; i < size; i++)
	data[2 * i + 1] = -data[2 * i + 1];

    return EDSP_OK;
}
