;
; FILE NAME:    iir.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        Iir
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the IIR filter.
;
;               int Iir(output, input, no_samples, coeff, no_sections,
;                       workspace)
;
; PARAMETERS:   short   output[]
;         const short   input[]
;               long    no_samples
;         const short   coeff[]
;               long    no_sections
;               short   *workspace
;

	.EXPORT	_Iir
	.IMPORT	SHT

; PROGRAM ENTRY POINT
;
	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_Iir:	MOV	@(0,R15),R1	;R9=NO_COEF
	MOV	@(4,R15),R3	;R3 -> WORKSPACE
	MOV	R9,@-R15
	MOV	R1,R9		;R9=NO_COEF
	MOV	R14,@-R15
	MOV	R7,R14		;R7=R14=COEF
	MOV	R13,@-R15
	MOV	R12,@-R15
	MOV	R11,@-R15
	MOV	R10,@-R15
	MOV	R9,R10		;R9=R10=N_SECTION
	MOV	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
;				;R4=OUTPUT
;				;R5=INPUT
;				;R6=NO

; Check parameters
	CMP/PL	R6		;NO >0 ?
	BF	ERR
	CMP/PL	R9		;N_SEC > 0 ?
	BT	argsOK
ERR:	BRA	RTN
	MOV	#1,R0

	.ALIGN	4
argsOK:	MOV.W	NONSAT,R2
	STC	SR,R1
	AND	R2,R1		;SET S BIT TO 0
	LDC	R1,SR		; ie set non-saturating mode

; Check that the biquad a0k's (ie result shift values) are valid
	MOV	R9,R1		;R1=NO OF SECTIONS
	MOV	#16,R2
SHTCHK:	MOV.W	@R14+,R0	;READ res_shift
	CMP/PZ	R0		;res_shift >= 0 ?
	BF	ERR		;ERROR IF NOT
	CMP/GT	R2,R0		;res_shift > 16 ?
	BT	ERR		;ERROR IF YES
	ADD	#10,R14
	ADD	#-1,R1
	CMP/PL	R1
	BT	SHTCHK

	MOV	R7,R14		;R7=R14=COEF
newsample:
	MOV.W	@R5+,R12	;READ INPUT SAMPLE
	MOV	R3,R1		;R1=R3=WORKSPACE

; The filter is computed one biquad at a time
newbiquad:
	MOV.W	@R14+,R0	;R0=res_shift
	CLRMAC			;CLEAR MAC
	MAC	@R3+,@R14+	;D(1)*A1
	SWAP.W	R12,R11
	MAC	@R3+,@R14+	;D(2)*A2
	EXTS.W	R11,R11		;R11=SIGN EXTENSION OF INPUT SAMPLE
	SHLL16	R12		;INPUT SAMPLE << 16
	STS	MACL,R8
	SHAR	R12		;INPUT SAMPLE >> 1
	STS	MACH,R13
	CLRT
	ADDC	R12,R8
	ADDC	R11,R13		;ADD INPUT TO PRODUCT
; Have calculated the first sum - now shift it by 15 bits.
	MOV.L	SHT15,R2
	JSR	@R2		;CALL SHIFT 15 ROUTINE
	CLRT

	MOV	R8,@-R15	;PUSH R8 ON STACK
	CLRMAC
	ADD	#2,R15
	MAC	@R15+,@R14+
	ADD	#-4,R3		;R3 -> D(1)
	MOV.W	@R3,R11		;R11=D(1)
	MAC	@R3+,@R14+	;D(1)*B1
	MAC	@R3+,@R14+	;D(2)*B2
	MOV.W	R11,@-R3	;SHIFT D(1) INTO D(2)
	MOV.W	R8,@-R3		;SHIFT D(0) INTO D(1)

;CALL SHIFTING
	STS	MACL,R8		;R8=MACL
	STS	MACH,R13
	SHLL2	R0		;R0=res_shift
	SHLL2	R0
	SHLL	R0		;R0 << 5
	MOV	SHTADDR,R2	;R2=ADDRESS OF SHIFTING ROUTINE
	ADD	R0,R2
	JSR	@R2		;CALL SHIFT ROUTINE
	CLRT

	MOV	R8,R12		;16LSB(R12)=INPUT FOR NEXT STAGE
	EXTS	R12,R12		;SIGN EXTENDED

	ADD	#4,R3		;R3 -> D(1) OF NEXT STAGE
	ADD	#-1,R10		;DECREASE STAGE COUNTER
	CMP/PL	R10
	BT	newbiquad

; Have now finished all biquads
	MOV.W	R12,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER
	MOV	R7,R14		;RESUME POINTER FOR COEF
	MOV	R1,R3		;RESUME POINTER FOR DELAY ELEMENT
	MOV	R9,R10		;RESUME STAGE COUNT
	ADD	#-1,R6		;DECREASE INPUT NO COUNT
	CMP/PL	R6
	BT	newsample

; Have now processed all samples
	MOV	#0,R0
RTN:	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	MOV	@R15+,R14
	RTS
	MOV	@R15+,R9

	.ALIGN	4
SHTADDR	.data.l	SHT
SHT15	.data.l	SHT+480
NONSAT:	.data.w	H'0FFFD

	.END
