/*
** FILE NAME:   IIRFILT.C
**
** PROJECT ID:  DSPLIB SH  
**
**
** AUTHOR:      D. Maidment
**
** DESCRIPTION: This file Generates a series of sine waves, applies an FFT 
**              to show the spectrum, then applies an FIR to remove all but one signal 
**              
**
** MODIFICATION HISTORY:
**
**              1.00    08-DEC-95
**
*/
#include <stdio.h>
#include <math.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "iircoeff.dat"

#define NSAM     128              /* number of samples */
#define TWOPI     6.283185307
#define no_coeff  12
#define no_stage  2
#define MAXSIZE   NSAM
#define NSAMS   (NSAM/2)
#define MAXSUM  67108864

/***************** Exercise To Take The FFT Of Data *******************/


void 
main()
{
        short sine1[NSAM], sine2[NSAM], sine3[NSAM], sine4[NSAM], sine5[NSAM], signal[NSAM], fftout1[NSAM], output1[NSAMS], filout[NSAM], fftout2[NSAM], output2[NSAMS];
        int     len, n, k, res_shift;
        float   scale;
        short   *work, *locate;
        long    sum;

        scale   =   1;
        len     =   NSAM;
        res_shift = 16;
        locate = coeff;
        sum     =   0;

/*** Check Scaling Of Coefficients ***/

        for(n = 0; n < no_coeff; n++)
        {
        sum += fabs(coeff[n]);
        }

        if(sum >= MAXSUM)
        printf("Bad Coefficients");

/*** Generate Sine Wave ***/

        if(InitFft( MAXSIZE) != EDSP_OK)
        printf("Problem With Memory Allocation");

        printf("Generating Sine Wave");
        k = len / 8;
        for(n = 0; n < len; n++)
        {
        sine1[n] = floor(20 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 16;
        for(n = 0; n < len; n++)
        {
        sine2[n] = floor(20 * sin(TWOPI * k * n / len) + 0.5);
        }

        k = len / 4;
        for(n = 0; n < len; n++)
        {
        sine3[n] = floor(20 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 32;
        for(n = 0; n < len; n++)
        {
        sine4[n] = floor(20 * sin(TWOPI * k * n / len) + 0.5);
        }

        k = len / 64;
        for(n = 0; n < len; n++)
        {
        sine5[n] = floor(20 * sin(TWOPI * k * n / len) + 0.5);
        }

        printf("Done");
        


/*** Create Signal To Be Filtered By Adding Sine waves ***/

        for(n = 0; n < len; n++)
        {
        signal[n] = (sine1[n] + sine2[n] + sine3[n] + sine4[n] + sine5[n]);
        }

/*** Perform IIR Filter Operation ***/

        if(InitIir(&work, no_stage) != EDSP_OK)
        printf("Problem With InitFir");

        if(Iir( filout, signal, NSAM, coeff, no_stage, work) != EDSP_OK) 
        printf("Problem With Iir Operation");



}


