/*
** FILE NAME:	initlms.c
**
** PROJECT ID:	94-0040
**
** TITLE:	InitLms
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	Routine to allocate and initialise
**              workspace memory for LMS filtering
**
**		int InitLms(workspace, no_coeffs)
** PARAMETERS:
**              short **workspace   : double pointer to filter memory
**              long no_coeffs      : number of coefficients K
**
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : no_coeffs <= 2
**              EDSP_NO_HEAP        : insufficient space available from
**                                    malloc.
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <stdio.h>
#include <stdlib.h>
#include <ensigdsp.h>

int 
InitLms(short **workspace, long no_coeff)
{
    /* Variable declaration */
    int i;
    long map;

    /* Input Argument Error checking */
    if (no_coeff <= 2)
	return EDSP_BAD_ARG;

    /* memory allocation */
    if ((*workspace = (short *)malloc((no_coeff + 2) * sizeof(short))) == NULL)	/* storage space
										 * checking */
	 return EDSP_NO_HEAP;

    /* workspace initialisation */
    for (i = 2; i < (no_coeff + 2); i++)
	(*workspace)[i] = 0;

    map = (long)&workspace[0][2];
    (*workspace)[1] = map & 0xFFFF;
    (*workspace)[0] = map >> 16;

    return EDSP_OK;		/* Return Success */
}
