;
; FILE NAME:    matrix.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        MatrixMult
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the matrix multiplication function.
;

	.EXPORT	_MatrixMult
	.IMPORT	SHT

;REGISTER USAGE
;
;R4 -> OUTPUT
;R5 -> MX1
;R6 -> MX2
;R7 -> NO_ROW1
;R9 -> NO_COL1
;R10-> NO_COL2

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_MatrixMult:	MOV.L	@R15+,R1	;READ NO_COL1
	MOV.L	@R15+,R2	;READ NO_COL2
	MOV.L	@R15+,R0	;R0=SHIFT
	ADD	#-12,R15
	MOV.L	R14,@-R15	;STORE REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV.L	R11,@-R15
	MOV.L	R10,@-R15
	MOV	R2,R10		;R10=NO_COL2
	MOV.L	R9,@-R15
	MOV	R1,R9		;R9=NO_COL1
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15

; Error checking
	CMP/PL	R7		;NO_ROW1 >0 ?
	BF	ERR
	CMP/PL	R9		;NO_COL1 >0 ?
	BF	ERR
	CMP/PL	R10		;NO_COL2 >0 ?
	BF	ERR
	CMP/PZ	R0		;SHIFT >=0 ?
	BF	ERR
	MOV	#27,R1
	CMP/GT	R1,R0		;SHIFT > 27 ?
	BF	ST1

ERR:	BRA	RTN
	MOV	#1,R0		;R0= RETURN CODE

	.ALIGN	4
ST1:	SHLL2	R0
	SHLL2	R0
	MOV	SHTADDR,R2	;R2=ADDRESS OF SHIFT ROUTINE
	SHLL	R0
	ADD	R2,R0		;R0=SHIFT ROUTINE ADDRESS

	MOV	R6,R3		;R3=R6 -> MX2
	MOV	R10,R14		;R14=R10=NO_COL2
	MOV	R14,R1
	SHLL	R1
	ADD	#-2,R1		;R1=2*NO_COL2 - 2
ST:	MOV	R9,R13		;R13=R9=COL1/ROW2 COUNT
	MOV	R5,R11		;R11 -> MX1
	MOV	R6,R2		;R2 -> MX2
	CLRMAC

	MOV	UPMASK,R12	;MASK AWAY UPPER 30 BITS
	AND	R13,R12		;R11=LOWER 2 BITS

LB5:	MOV	LBADDR,R8	;R8=ADDRESS OF LB
	SHLL2	R12		;R12*4
	SUB	R12,R8		;R8=ADDRESS OF LB - R12*2

	MOV	R13,R12
	SHLL	R12

	JMP	@R8		;R13=JUMP ADDRESS
	ADD	R11,R12		;LOOP END ADDRESS

	.ALIGN	4
LBADDR	.data.l	LB
	NOP
LB7:	MAC.W	@R11+,@R2+
	ADD	R1,R2
	MAC.W	@R11+,@R2+
	ADD	R1,R2
	MAC.W	@R11+,@R2+
	ADD	R1,R2
	MAC.W	@R11+,@R2+
	ADD	R1,R2
LB:	CMP/EQ	R12,R11
	BF	LB7

LB6:	STS.L	MACH,R13
	STS.L	MACL,R8

;CALL SHIFT
	JSR	@R0		;CALL SHIFT ROUTINE
	CLRT

	ADD	#2,R6		;INCREASE MX2 POINTER
	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE OUTPUT POINTER

	ADD	#-1,R10		;DECREASE COL2
	CMP/PL	R10
	BT	ST

	MOV	R14,R10		;RESET R10
	MOV	R3,R6		;RESET R6
	ADD	R9,R5		;R5+2*R9
	ADD	R9,R5
	ADD	#-1,R7		;DECREASE ROW1
	CMP/PL	R7
	BT	ST


RTN3:	MOV	#0,R0		;RETURN CODE
RTN:	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14


	.ALIGN	4
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC

	.END

