/*
** FILE NAME:	matrix.c
**
** PROJECT ID:	94-0040
**
** TITLE:	MatrixMult
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine multiplies the two given matrices
**
**		int MatrixMult(op_matrix, ip_matrix1, ip_matrix2,  no_rows1,
**                         no_cols1, no_cols2, res_shift)
** PARAMETERS:
**              void *op_matrix    : pointer to first element of output matrix.
**              void *ip_matrix1   : pointer to first element of input matrix1.
**              void *ip_matrix2   : pointer to first element of input matrix2.
**              long no_row1       : row dimension of ip_matrix1, m.
**              long no_cols1      : column dimension of ip_matrix1, row dimension
**                                   of matrix 2, n.
**              long no_cols2      : column dimension of ip_matrix2, p.
**              int res_shift      : right shift applied to each output
**
**
** RETURNS:     EDSP_OK               : success
**              EDSP_BAD_ARG          : no_rows1  < 1
**                                      no_cols1  < 1
**                                      no_cols2  < 1
**                                      res_shift < 0
**                                      res_shift > 27
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int 
MatrixMult(void *op_matrix,
	   const void *ip_matrix1,
	   const void *ip_matrix2,
	   long no_rows1,
	   long no_cols1,
	   long no_cols2,
	   int res_shift)
{
    /* Variable Declaration */
    int i, j, k;
    double temp, temp1;

    /* Pointer copying */
    short *matrix11 = (short *)ip_matrix1;
    short *matrix21 = (short *)ip_matrix2;
    short *matrix31 = (short *)op_matrix;

    /* Input Argument Error Checking */
    if ((no_rows1 < 1) || (no_cols1 < 1) || (no_cols2 < 1) || (res_shift < 0) || (res_shift > 27))
	return EDSP_BAD_ARG;

    for (i = 0; i < no_rows1; i++)	/* Row Loop */
    {
	for (j = 0; j < no_cols2; j++)	/* Column Loop */
	{
	    temp = 0;		/* Variable Initialisation */
	    for (k = 0; k < no_cols1; k++)
	    {
		temp += (double)matrix11[no_cols1 * i + k] *
		    matrix21[k * no_cols2 + j];

	    }

	    /* Right shifting and rounding */
	    temp1 = floor(temp / (1L << res_shift) + 0.5);

	    /* Saturation */
	    if (temp1 > 32767)
		temp1 = 32767;
	    else if (temp1 < -32768)
		temp1 = -32768;
	    *matrix31 = temp1;	/* store in output */
	    matrix31++;
	}
    }

    return EDSP_OK;		/* Return Success */

}
