;
; FILE NAME:    mult.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        MULT, DADD
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the 31bit x 31bit multiply and 64-bit
;		addition functions. They are used by the double precision IIR
;		filter functions, but should not be used by user programs.
;
;		For the multiply (MULT):
;			The source operands are taken from R1 and R2 - the LSB
;			of each register is ignored in this calculation.
;			The result is returned in (R1:R2).
;			Registers R0, R8, R11, R12 and R13 are used as scratch
;			registers.
;
;		For the addition (DADD):
;			The source operands are (R1,R2) and (R7,R9).
;			The result is returned in (R7,R9).
;			No other scratch registers are used.
;

	.EXPORT	MULT
	.EXPORT	DADD

	.SECTION	P,CODE,ALIGN=4

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Double precision multiply
; Calculate (R1,R2) = R1 * R2
MULT:
	; Extract the source MSB's into separate registers
	MOV	R1, R8
	SHLR16	R8		; r8 = src1h = src1[31:16]
	MOV	R2, R11
	SHLR16	R11		; r11 = src2h = src2[31:16]
	; Discard the source LSB's.
	SHLR	R1		; r1 = src1l >> 1 = src1[15:1]
	SHLR	R2		; r2 = src2l >> 1 = src2[15:1]
	; Clear the LSB sign bits
	MOV.W	BITS14, R0
	AND	R0, R1
	AND	R0, R2

	; Perform the four partial multiplies
	MULU	R1, R2
	STS	MACL, R0	; r0 = src1[15:1] * src2[15:1]
	MULS	R11, R1
	SHLL2	R0
	STS	MACL, R12	; r12 = src1[15:1] * src2[31:16]
	MULS	R8, R2
	STS	MACL, R13	; r13 = src1[31:16] * src2[15:1]

	; First combine src1h.srchl and src1l.src2h.
	; They both have 2 sign bits, so overflow cannot occur.

	ADD	R12, R13
	MULS	R8, R11
	MOV	R13, R2
	STS	MACL, R8	; r8 = src1[31:16] * src2[31:16]
	SHLL16	R2
	SHLL	R2

;do r13 >> 15
	SWAP	R13,R13
	ROTL	R13
	ROTR	R13		;copy the msb to T
	EXTS.W	R13,R1
	ROTCL	R1		;T => lsb

	; Now have 4 sources for the final summation:
	;	r0 = bits [31:0] (bit 0 is always 0)
	;	r11 = bits [63:32] (bits 63 and 62 are sign)
	;	r1 = bits [31:16]
	;	r13 = bits [47:32]

	CLRT
	ADDC	R0, R2		; r2 = bits [31:0] of result
	ADDC	R8, R1		; r1 = bits [64:32] of result

;SHIFT << 1 TO GET RID OF REDUNDANT SIGN BIT
	MOV.L	MIN,R8
	CMP/EQ	R8,R1		;trap -1 * -1
	BF	RTN
	MOV.L	POSL,R2
	MOV.L	POS,R1		;return +ve saturation
RTN:	RTS
	NOP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Double precision addition - 
; Calculate  (R7,R9)=(R1,R2)+(R7,R9) with saturation.
;
	.ALIGN	4
DADD:	CLRT
	ADDC	R2,R9		;SUM 32 LSB
	BF	NOC		;IF NO CARRY, GOTO NOC
	ADDV	R1,R7		;SUM MSB'S
	BT	SAT1		;OVERFLOW ?
	MOV	#1,R2
	ADDV	R2,R7		;NO, ADD CARRY TO MSB'S
	BF	RTN1		;OVERFLOW ?
SAT1:	MOV	R7,R1
	MOV.L	POS1,R7		;POSITIVE SATURATION
	MOV.L	POSL,R9
	CMP/PZ	R1
	BF	RTN1
	MOV.L	NEG,R7		;NEGATIVE SATURATION
	RTS
	MOV	#0,R9

NOC:	ADDV	R1,R7		;SUM MSB'S
	BT	SAT1		;OVERFLOW ?

RTN1:	RTS
	NOP

	.align 4
CARRY	.data.l	H'00010000
POS	.data.l	H'3FFFFFFF
POS1	.data.l	H'7FFFFFFF
POSL	.data.l	h'FFFFFFFF
NEG	.data.l H'80000000
MIN	.data.l	h'40000000
BITS14:	.data.w	H'7fff

	.end
