/*
** FILE NAME:	tconvpar.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB ConvPartial() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tconvpar() checks that ConvPartial():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Correctly flags incorrect arguments
**		d) Passes a bit exact test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	NX  	5
#define	NW  	5
#define	NOUT	(2*NX-NW)*(NW+1)/2
#define MAXSH	27
#define	NARGS	6

void 
Tconvpar(char *rcode)
{
    short i, j, n, errc;
    short output[NOUT];
    int xsize, wsize, rshift;
    long sum;

    static int stkchk[NARGS];

#include "tconvpar.txt"

/* test working of stack */
    xsize = NX;
    wsize = NW;
    rshift = 15;
    stkchk[0] = (int)output;
    stkchk[1] = (int)w;
    stkchk[2] = (int)x;
    stkchk[3] = wsize;
    stkchk[4] = xsize;
    stkchk[5] = rshift;

    /* also test for OK return argument */
    if (ConvPartial(output, w, x, wsize, xsize, rshift) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)w ||
	stkchk[2] != (int)x ||
	stkchk[3] != wsize ||
	stkchk[4] != xsize ||
	stkchk[5] != rshift)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test vector 1 */
    errc = 0;
    for (i = 0; i < NX - NW; i++)
	if (output[i] != t1[i])
	{
	    sprintf(rcode, "%d %d,", output[i], t1[i]);
	    errc++;
	}
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);

/* test return args */
    for (xsize = -2; xsize < 1; xsize++)
	if (ConvPartial(output, w, x, NW, xsize, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (wsize = -2; wsize < 1; wsize++)
	if (ConvPartial(output, w, x, wsize, NX, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    if (ConvPartial(output, w, x, NW, NW - 1, 0) != EDSP_BAD_ARG)
	sprintf(rcode, "Bad Return Argument\n");
    for (rshift = -2; rshift < 0; rshift++)
	if (ConvPartial(output, w, x, NW, NX, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = 28; rshift < 45; rshift++)
	if (ConvPartial(output, w, x, NW, NX, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

    for (i = 0; i < NOUT; output[i++] = 0) ;
    j = 0;
    errc = 0;
    for (wsize = 1; wsize <= NW; wsize++)
	for (xsize = wsize; xsize <= NX; xsize++)
	    for (rshift = 0; rshift <= MAXSH; rshift++)
	    {
		if (ConvPartial(output, w, x, wsize, xsize, rshift) != EDSP_OK)
		    sprintf(rcode, "EDSP_OK not returned\n");
		sum = 0;
		for (n = 0; n <= xsize - wsize; n++)
		    sum += output[n];
		if (sum != sumconvpar[j])
		{
		    errc++;
		}
		j++;
	    }
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "ConvPartial Passed\n");
    else
	strcat(rcode, "ConvPartial Failed\n");
}
