/*
** FILE NAME:   textr.c
**
** PROJECT ID:  94-0040
**
** TITLE:       File that tests the DSPLIB MaxI(), MinI() and PeakI() functions
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: The function Textr() checks that each of
**              MaxI(), MinI() and PeakI()
**              a) Does not corrupt the stack
**              b) Passes a simple test vector
**              c) Correctly flags incorrect arguments
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "miscdat.txt"
#include "textr.txt"


#define N       100
#define NARGS   3

void 
Textr(char *rcode)
{
    short *outp, **outpp;
    int size;

    static int stkchk[NARGS];


/* MAXI */
/* test working of stack */
    size = N;
    outpp = &outp;
    stkchk[0] = (int)outpp;
    stkchk[1] = (int)dat;
    stkchk[2] = size;

    /* also test for OK return argument */
    if (MaxI(outpp, dat, size) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)outpp ||
	stkchk[1] != (int)dat ||
	stkchk[2] != size)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

    if (outp != dat + maxi)
	sprintf(rcode, "Maximum index not returned - %d\n", *outp);

/* test return args */
    for (size = -2; size < 1; size++)
	if (MaxI(outpp, dat, size) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

/* test small args */
    if (MaxI(outpp, dat, 1) != EDSP_OK)
	sprintf(rcode, "Bad Return Argument\n");
    if (*outp != dat[0])
	sprintf(rcode, "Maximum not returned - %d\n", *outp);

/* MINI */
/* test working of stack */
    size = N;
    outpp = &outp;
    stkchk[0] = (int)outpp;
    stkchk[1] = (int)dat;
    stkchk[2] = size;

    /* also test for OK return argument */
    if (MinI(outpp, dat, size) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)outpp ||
	stkchk[1] != (int)dat ||
	stkchk[2] != size)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

    if (outp != dat + mini)
	sprintf(rcode, "Minimum index not returned - %d\n", *outp);

/* test return args */
    for (size = -2; size < 1; size++)
	if (MinI(outpp, dat, size) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

/* test small args */
    if (MinI(outpp, dat, 1) != EDSP_OK)
	sprintf(rcode, "Bad Return Argument\n");
    if (*outp != dat[0])
	sprintf(rcode, "Minimum not returned - %d\n", *outp);

/* PEAKI */
/* test working of stack */
    size = N;
    outpp = &outp;
    stkchk[0] = (int)outpp;
    stkchk[1] = (int)dat;
    stkchk[2] = size;

    /* also test for OK return argument */
    if (PeakI(outpp, dat, size) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)outpp ||
	stkchk[1] != (int)dat ||
	stkchk[2] != size)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

    if (outp != dat + peaki)
	sprintf(rcode, "Peak index not returned - %d \n", *outp);

/* test return args */
    for (size = -2; size < 1; size++)
	if (PeakI(outpp, dat, size) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

/* test small args */
    if (PeakI(outpp, dat, 1) != EDSP_OK)
	sprintf(rcode, "Bad Return Argument\n");
    if (*outp != dat[0])
	sprintf(rcode, "Peak not returned - %d\n", *outp);

    if (strlen(rcode) == 0)
	sprintf(rcode, "MaxI,MinI,PeakI Passed\n");
    else
	strcat(rcode, "MaxI,MinI,PeakI Failed\n");
}
