/*
** FILE NAME:   tmatrixm.c
**
** PROJECT ID:  94-0040
**
** TITLE:       File that tests the DSPLIB MatrixMult() function
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: The function Tmatrixm() checks that MatrixMult()
**              a) Does not corrupt the stack
**              b) Correctly flags incorrect arguments
**              c) Passes a bit-exact test
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "tmatrixm.txt" 

#define N       4
#define NN      N*N
#define MAXSH   27
#define NARGS   7

void 
main()
{
    short i, j, errc;
    short output[NN];
    int m, n, p, rshift;
    long sum;

    static int stkchk[NARGS];


	printf("Executing Tmatrixm.c now........");
/* test working of stack */
    m = n = p = N;
    rshift = 15;
    stkchk[0] = (int)output;
    stkchk[1] = (int)m1;
    stkchk[2] = (int)m2;
    stkchk[3] = m;
    stkchk[4] = n;
    stkchk[5] = p;
    stkchk[6] = rshift;

    /* also test for OK return argument */
    if (MatrixMult(output, m1, m2, m, n, p, rshift) != EDSP_OK)
	sprintf("EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)m1 ||
	stkchk[2] != (int)m2 ||
	stkchk[3] != m ||
	stkchk[4] != n ||
	stkchk[5] != p ||
	stkchk[6] != rshift)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test return args */
    for (m = -2; m < 1; m++)
	if (MatrixMult(output, m1, m2, m, N, N, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - m\n");
    for (n = -2; n < 1; n++)
	if (MatrixMult(output, m1, m2, N, n, N, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - n\n");
    for (p = -2; p < 1; p++)
	if (MatrixMult(output, m1, m2, N, N, p, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - p\n");
    for (rshift = -2; rshift < 0; rshift++)
	if (MatrixMult(output, m1, m2, N, N, N, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift\n");
    for (rshift = 28; rshift < 45; rshift++)
	if (MatrixMult(output, m1, m2, N, N, N, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift\n");


/****** Insert My Code Here ********************************************/
	printf("Start 4*4 matix mult");
	m = n = p = 4;
	rshift  =   0;
	MatrixMult(output, m1, m2, m, n, p, rshift);
	printf("Done");
/**********************************************************************/

    for (i = 0; i < NN; output[i++] = 0) ;
    j = 0;
    errc = 0;
    for (n = 1; n <= N; n++)
	for (m = 1; m <= N; m++)
	    for (p = 1; p <= N; p++)
		for (rshift = 0; rshift <= MAXSH; rshift += 4)
		{
		    if (MatrixMult(output, m1, m2, m, n, p, rshift) != EDSP_OK)
			sprintf(rcode, "EDSP_OK not returned\n");
		    sum = 0;
		    for (i = 0; i < m * p; i++)
		    {
			sum += output[i];
		    }
		    if (sum != summatm[j])
		    {
			errc++;
		    }
		    j++;
		}
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "MatrixMult Passed\n");
    else
	strcat(rcode, "MatrixMult Failed\n");
}
