/*
** FILE NAME:   tmspower.c
**
** PROJECT ID:  94-0040
**
** TITLE:       File that tests the DSPLIB MsPower() function
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: The function Tmspower() checks that MsPower()
**              a) Does not corrupt the stack
**              b) Correctly flags incorrect arguments
**              c) Passes a bit-exact test
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/

#include <stdio.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "miscdat.txt"
#include "tmspower.txt"


#define N       100
#define NARGS   3

void 
Tmspower(char *rcode)
{
    int i, errc;
    long size, output[1];

    static int stkchk[NARGS];


/* test working of stack */
    size = N;
    stkchk[0] = (int)output;
    stkchk[1] = (int)dat;
    stkchk[2] = size;

    /* also test for OK return argument */
    if (MsPower(output, dat, size) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)dat ||
	stkchk[2] != size)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test return args */
    for (size = -2; size < 1; size++)
	if (MsPower(output, dat, size) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

    errc = 0;
    for (i = 0; i < N; i++)
    {
	if (MsPower(output, dat, i + 1) != EDSP_OK)
	    sprintf(rcode, "EDSP_OK not returned\n");
	if (*output != pow[i])
	    errc++;
    }
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "MsPower Passed\n");
    else
	strcat(rcode, "MsPower Failed\n");
}
