/*
** FILE NAME:	tstiir.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB Iir() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tstiir() checks that Iir():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Handles small array sizes
**		d) Correctly flags incorrect arguments
**		e) Passes a bit exact sinusoid filtering test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	K  	2
#define	N  	100
#define NS	30
#define MAXSH	16
#define	NARGS	6

void 
Tstiir(char *rcode)
{
    short *work, i, j, k, rshift, errc;
    short output[N];
    short coeff2[6 * K];
    int   nsamp, nsect;
    long  sum;

#include "tstiir.txt"
#include "filtdat.txt"
    static int stkchk[NARGS];

    if (InitIir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");

/* test working of stack */
    nsamp = 10;
    nsect = K;
    stkchk[0] = (int)output;
    stkchk[1] = (int)input;
    stkchk[2] = nsamp;
    stkchk[3] = (int)coeff;
    stkchk[4] = nsect;
    stkchk[5] = (int)work;

    /* also test for OK return argument */
    if (Iir(output, input, nsamp, coeff, nsect, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");

    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)input ||
	stkchk[2] != nsamp ||
	stkchk[3] != (int)coeff ||
	stkchk[4] != nsect ||
	stkchk[5] != (int)work)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* continue with test vector 1  - tests for correct workspace usage */
    if (Iir(output+nsamp, input+nsamp, N-nsamp, coeff, nsect, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    free(work);
    errc = 0;
    for (i = 0; i < NS; i++)
    {
	if (output[i] != t1[i])
	    errc++;
    }
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);

/* test return args */
    if (InitIir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");
    for (nsamp = -2; nsamp <= 0; nsamp++)
	if (Iir(output, input, nsamp, coeff, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - nsamp\n");
    for (nsect = -2; nsect <= 0; nsect++)
	if (Iir(output, input, N, coeff, nsect, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - nsect\n");
    for (i = 0; i < 6 * K; i++)
	coeff2[i] = coeff[i];
    for (rshift = -2; rshift < 0; rshift++)
    {
	coeff2[0] = rshift;
	if (Iir(output, input, N, coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift\n");
    }
    for (rshift = 17; rshift < 20; rshift++)
    {
	coeff2[0] = rshift;
	if (Iir(output, input, N, coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift\n");
    }
    coeff2[0] = 0;
    for (rshift = -2; rshift < 0; rshift++)
    {
	coeff2[6] = rshift;
	if (Iir(output, input, N, coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift\n");
    }
    for (rshift = 17; rshift < 20; rshift++)
    {
	coeff2[6] = rshift;
	if (Iir(output, input, N, coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift\n");
    }
    free(work);

/* test small array sizes */
    /* small number of inputs */
    for (i = 0; i < N; output[i++] = 0) ;
    nsect = K;
    errc = 0;
    for (nsamp = 1; nsamp < 5; nsamp++)
    {
	if (InitIir(&work, K) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	Iir(output, input, nsamp, coeff, nsect, work);
	if (output[nsamp - 1] != t1[nsamp - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T1: %d errors\n", errc);

/*	filter sinusoids at frequencies up to 0.5*Fs in steps of 0.05*Fs
	and compare with precalculated data */
    j = 0;
    errc = 0;
    for (k = 0; k <= N / 2; k += N / 20)
    {
	for (rshift = 0; rshift <= MAXSH; rshift++)
	{
	    if (InitIir(&work, K) != EDSP_OK)
		sprintf(rcode, "Init Problem\n");
	    coeff[0] = rshift;
	    coeff[6] = rshift / 2;
	    Iir(output, testdat[j], N, coeff, K, work);;
	    sum = 0;
	    for (i = 0; i < N; i++)
		sum += output[i];
	    if (sum != sumiir[j][rshift])
		errc++;
	    free(work);
	}
	j++;
    }
    if (errc)
	sprintf(rcode, "Failed Sinusoid Test: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "Iir Passed\n");
    else
	strcat(rcode, "Iir Failed\n");
}
