/*
** FILE NAME:   tvectm.c
**
** PROJECT ID:  94-0040
**
** TITLE:       File that tests the DSPLIB VectorMult() function
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: The function Tvectm() checks that VectorMult()
**              a) Does not corrupt the stack
**              b) Correctly flags incorrect arguments
**              c) Passes a bit-exact test
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/

#include <stdio.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "tvectm.txt"

#define N       4
#define MAXSH   16
#define NARGS   5
#define RSHIFT  15

void 
Tvectm(char *rcode)
{
    short i, j, n, errc;
    short output[N];
    int size, rshift;

    static int stkchk[NARGS];


	printf("testing Stack");
/* test working of stack */
    size = N;
    rshift = RSHIFT;
    stkchk[0] = (int)output;
    stkchk[1] = (int)w;
    stkchk[2] = (int)x;
    stkchk[3] = size;
    stkchk[4] = rshift;

    /* also test for OK return argument */
    if (VectorMult(output, w, x, size, rshift) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)w ||
	stkchk[2] != (int)x ||
	stkchk[3] != size ||
	stkchk[4] != rshift)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test return args */
	printf("test Bad Arguments");
    for (size = -2; size < 1; size++)
	if (VectorMult(output, w, x, size, RSHIFT) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = -2; rshift < 0; rshift++)
	if (VectorMult(output, w, x, N, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = 17; rshift < 45; rshift++)
	if (VectorMult(output, w, x, N, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

    for (i = 0; i < N; output[i++] = 0) ;
    j = 0;
    errc = 0;
    printf("bit exact test");
    for (size = 1; size <= N; size++)
	for (rshift = 0; rshift <= MAXSH; rshift++)
	{
	    if (VectorMult(output, w, x, size, rshift) != EDSP_OK)
		sprintf(rcode, "EDSP_OK not returned\n");
	    for (n = 0; n < size; n++)
	    {
		if (output[n] != vectm[j])
		    errc++;
		j++;
	    }
	}
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "VectorMult Passed\n");
    else
	strcat(rcode, "VectorMult Failed\n");
}
