/*
** FILE NAME:   vectmult.c
**
** PROJECT ID:  94-0040
**
** TITLE:       VectorMult
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: This routine multiplies two vectors term by term
**
**              int VectorMult(output, ip1, ip2, no_elements, res_shift)
**
** PARAMETERS:
**              short output[]      : output
**              short ip1[]         : input 1
**              short ip2[]         : input 2
**              long no_elements    : number of elements to evaluate
**              int res_shift       : right shift applied to each output
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : no_elements < 1
**                                    res_shift < 0
**                                    res_shift > 16
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int 
VectorMult(short output[],
	   const short ip1[],
	   const short ip2[],
	   long no_elements,
	   int res_shift)
{
    /* Variable Declaration */
    int i;
    double temp;

    /* Input Argument Error Checking */
    if ((no_elements < 1) || (res_shift < 0) || (res_shift > 16))
	return EDSP_BAD_ARG;

    for (i = 0; i < no_elements; i++)   /* Main Loop */
    {
	temp = (double)ip1[i] * ip2[i];
	temp = floor(temp / (1L << res_shift) + 0.5);

	if (temp > 32767)
	    temp = 32767;
	else if (temp < -32768)
	    temp = -32768;

	output[i] = temp;
    }

    return EDSP_OK;             /* Return Success */
}
